using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Text;
using System.Windows.Forms;

// Steema namespaces
using Steema.TeeChart;
using Steema.TeeChart.Styles;

namespace ContourWaterfall
{
    public partial class Form1 : Form
    {
        /// <summary>
        /// Axis titles
        /// </summary>
        private const string XAXIS_TITLE = "Frequency";
        private const string YAXIS_TITLE = "Magnitude";
        private const string ZAXIS_TITLE = "Time";

        /// <summary>
        /// Time range 1
        /// </summary>
        public const int _lowerTimeLimit1 = 0;
        public const int _upperTimeLimit1 = 800;   ///400;

        /// <summary>
        /// Time range 2
        /// </summary>
        public const int _lowerTimeLimit2 = 0;
        public const int _upperTimeLimit2 = 801;    ///802;    ///805;    ///810;   ///825; ///850;   ///900;  ///1000;  ///1200;  ///1600;

        /// <summary>
        /// Time range 1 and 2 step
        /// </summary>
        public const int _timeStep = 1;

        /// <summary>
        /// Frequency range and step
        /// </summary>
        public const double _lowerFrequencyLimit = 0.0;
        public const double _upperFrequencyLimit = 200.0;
        public const double _frequencyStep = 0.25;

        /// <summary>
        /// The number of colours in the palatte.
        /// </summary>
        public const int PALETTEENTRIES = 128;

        /// <summary>
        /// Countour plot 1 values, X = _frequency1, Y = _value1, Z = _time1
        /// </summary>
        private double[] _time1;
        private double[] _value1;
        private double[] _frequency1;

        /// <summary>
        /// Countour plot 2 values, X = _frequency2, Y = _value2, Z = _time2
        /// </summary>
        private double[] _time2;
        private double[] _value2;
        private double[] _frequency2;

        /// <summary>
        /// Contour plot 1 chart, data series and mark tip objects.
        /// </summary>
        private Steema.TeeChart.TChart _plotChartContour1;
        private Steema.TeeChart.Styles.ColorGrid _dataSeriesContour1;
        private Steema.TeeChart.Tools.MarksTip _markTipContour1;

        /// <summary>
        /// Contour plot 2 chart, data series and mark tip objects.
        /// </summary>
        private Steema.TeeChart.TChart _plotChartContour2;
        private Steema.TeeChart.Styles.ColorGrid _dataSeriesContour2;
        private Steema.TeeChart.Tools.MarksTip _markTipContour2;
        
        /// <summary>
        /// Entry point:
        ///     1.  Do forms initialisation;
        ///     2.  Generate data for the contour plots;
        ///     3.  Display the data.
        /// </summary>
        public Form1()
        {
            // Forms initialisation
            InitializeComponent();

            // Generate data for the two contour plots
            generateDataContour1(432);
            generateDataContour2(432);

            // MAke the form fit the screen
            this.SetBounds(SystemInformation.WorkingArea.X, SystemInformation.WorkingArea.Y, SystemInformation.WorkingArea.Width, SystemInformation.WorkingArea.Height);

            // Display the data
            displayContour1();
            displayContour2();
        }


        #region Contour Plot data generation operations
        /// <summary>
        /// Generate a data set with one vertical, maximum value, and one horizontal line at
        /// half maximum value, all other values are 0.
        /// </summary>
        private void generateDataContour1(int numberRows)
        {
            //
            // Local Declarations
            //
            int numberOfDataPoints;
            int pointPos;
            double magnitude;

            //
            // Start of code
            //
            pointPos = 0;

            // Calculate the number of data points based on the frequency range, X, and step and
            // time range, Z, and step. Note all ranges are 0 to an upperlimit.
            numberOfDataPoints = (int)_upperTimeLimit1 * (int)_upperFrequencyLimit * (int)(1.0 / _frequencyStep) * (int)(1.0 / _timeStep);

            // Create the X = _frequency1, Y = _value1, Z = _time1 arrays
            _time1 = new double[numberOfDataPoints];
            _value1 = new double[numberOfDataPoints];
            _frequency1 = new double[numberOfDataPoints];

            // Create the data points loop by time, Z, and then frequency, X.
            for (int time = _lowerTimeLimit1; time < _upperTimeLimit1; time = time + _timeStep)
            {
                for (double frequency = _lowerFrequencyLimit; frequency < _upperFrequencyLimit; frequency = frequency + _frequencyStep)
                {
                    // Set point to 0.
                    magnitude = 0.0;

                    // Draw a vertical line of maximum value between 50.0 and 52.0.
                    if ((frequency >= 50.0) && (frequency <= 52.0))
                    {
                        magnitude = 7.5;
                    }

                    // Draw a horizontal line of half maximum value between half upper time limit
                    // +/- 2..
                    if ((time <= (_upperTimeLimit1 / 2) + 2) && (time >= (_upperTimeLimit1 / 2) - 2))
                    {
                        magnitude = 3.75;
                    }

                    // Save generated value and its X and Z position and move to the next data point position
                    _time1[pointPos] = time;
                    _value1[pointPos] = magnitude;
                    _frequency1[pointPos] = frequency;
                    pointPos = pointPos + 1;
                }
            }
        }

        /// <summary>
        /// Generate a data set with two vertical lines at maiximum value and one horizontal
        /// line at 2/3 maximum value and one at 1/3 maximum value.
        /// </summary>
        private void generateDataContour2(int numberRows)
        {
            //
            // Local Declarations
            //
            int numberOfDataPoints;
            int pointPos;
            double magnitude;

            //
            // Start of code
            //
            pointPos = 0;

            // Calculate the number of data points based on the frequency range, X, and step and
            // time range, Z, and step. Note all ranges are 0 to an upperlimit.
            numberOfDataPoints = (int)_upperTimeLimit2 * (int)_upperFrequencyLimit * (int)(1.0 / _frequencyStep) * (int)(1.0 / _timeStep);

            // Create the X = _frequency2, Y = _value2, Z = _time2 arrays
            _time2 = new double[numberOfDataPoints];
            _value2 = new double[numberOfDataPoints];
            _frequency2 = new double[numberOfDataPoints];

            // Create the data points loop by time, Z, and then frequency, X.
            for (int time = _lowerTimeLimit2; time < _upperTimeLimit2; time = time + _timeStep)
            {
                for (double frequency = _lowerFrequencyLimit; frequency < _upperFrequencyLimit; frequency = frequency + _frequencyStep)
                {
                    // Set point to 0.
                    magnitude = 0.0;

                    // Draw a vertical line of maximum value between 50.0 and 52.0.
                    if ((frequency >= 50.0) && (frequency <= 52.0))
                    {
                        magnitude = 15.0;
                    }

                    // Draw a vertical line of maximum value between 150.0 and 152.0.
                    if ((frequency >= 150.0) && (frequency <= 152.0))
                    {
                        magnitude = 15.0;
                    }

                    // Draw a horizontal line of 2/3 maximum value between half upper time limit
                    // +/- 2..
                    if ((time <= (_upperTimeLimit2 / 2) + 4) && (time >= (_upperTimeLimit2 / 2) - 4))
                    {
                        magnitude = 10.0;
                    }

                    // Draw a horizontal line of 1/3 maximum value between third upper time limit
                    // +/- 2..
                    if ((time <= (_upperTimeLimit2 / 3) + 4) && (time >= (_upperTimeLimit2 / 3) - 4))
                    {
                        magnitude = 5.0;
                    }

                    // Save generated value and its X and Z position and move to the next data point position
                    _time2[pointPos] = time;
                    _value2[pointPos] = magnitude;
                    _frequency2[pointPos] = frequency;
                    pointPos = pointPos + 1;
                }
            }
        }
        #endregion

        #region Contour Plot display operations
        /// <summary>
        /// Display contour plot 1
        /// </summary>
        private void displayContour1()
        {
            //
            // Local Declarations
            //

            //
            // Start of code
            //
            // Create the TChart
            _plotChartContour1 = new Steema.TeeChart.TChart();

            // Create the contour plot, create the Contour series
            _dataSeriesContour1 = new ColorGrid();

            // Turn off the data marks
            _dataSeriesContour1.Marks.Symbol.Shadow.Height = 1;
            _dataSeriesContour1.Marks.Symbol.Shadow.Visible = true;
            _dataSeriesContour1.Marks.Symbol.Shadow.Width = 1;
            _dataSeriesContour1.Marks.Visible = false;

            // Set the seiries title.
            _dataSeriesContour1.Title = "contourSeries";

            // Contour X, Y and Z Values
            _dataSeriesContour1.XValues.DataMember = "X";
            _dataSeriesContour1.YValues.DataMember = "Y";
            _dataSeriesContour1.ZValues.DataMember = "Z";

            // Add the data marks tip tool create the markstip
            _markTipContour1 = new Steema.TeeChart.Tools.MarksTip();

            // Add ,arkstip tool to the chart.
            _plotChartContour1.Tools.Add(_markTipContour1);

            // Set the style for label values
            _markTipContour1.Style = MarksStyles.PointIndex;
            _markTipContour1.MouseAction = Steema.TeeChart.Tools.MarksTipMouseAction.Click;
            _markTipContour1.MouseDelay = 100;
            _markTipContour1.HideDelay = 15000;

            // Make the waterfall series the one assiocated with the mark tip
            _markTipContour1.Series = _dataSeriesContour1;

            // Set the mark tip event handler.
            _markTipContour1.GetText += new Steema.TeeChart.Tools.MarksTipGetTextEventHandler(contour1MarkTipGetText);

            // Set the Axis for automatic scalling
            _plotChartContour1.Axes.Left.Automatic = true;
            _plotChartContour1.Axes.Left.AdjustMaxMin();
            _plotChartContour1.Axes.Bottom.Automatic = true;
            _plotChartContour1.Axes.Bottom.AdjustMaxMin();
            _plotChartContour1.Axes.Depth.Automatic = true;
            _plotChartContour1.Axes.Depth.AdjustMaxMin();
            _plotChartContour1.Axes.DepthTop.Automatic = true;

            // Set to fill available space
            _plotChartContour1.Dock = DockStyle.Fill;

            // Make the contour plot a 2D one.
            _plotChartContour1.Aspect.View3D = false;

            // Set the drawing mode
            _plotChartContour1.Aspect.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.HighQuality;
            _plotChartContour1.Aspect.TextRenderingHint = System.Drawing.Text.TextRenderingHint.ClearTypeGridFit;

            // Set the chart title
            _plotChartContour1.Header.Text = "Contour Plot 1: " + _time1.Length + " Data Points";

            // Set the Y (left) axis.
            _plotChartContour1.Axes.Left.Title.Text = ZAXIS_TITLE;

            // Set up the X (bottom) axis
            _plotChartContour1.Axes.Bottom.Labels.Font.Shadow.Visible = false;
            _plotChartContour1.Axes.Bottom.Labels.Shadow.Visible = true;

            // Set the X (bottom) axis caption
            _plotChartContour1.Axes.Bottom.Title.Text = XAXIS_TITLE;
            _plotChartContour1.Axes.Bottom.Title.Font.Shadow.Visible = false;
            _plotChartContour1.Axes.Bottom.Title.Font.Unit = System.Drawing.GraphicsUnit.World;
            _plotChartContour1.Axes.Bottom.Title.Shadow.Visible = false;

            // Add sample data to plot
            _plotChartContour1.Series.Add(_dataSeriesContour1);

            // Add the data with gaps to the data series
            _dataSeriesContour1.Add(_frequency1, _value1, _time1);

            // Set for irregularGrid
            _dataSeriesContour1.IrregularGrid = true;
            _dataSeriesContour1.CenteredPoints = false;

            // Set the axis minimum and maximum values
            _plotChartContour1.Axes.Bottom.SetMinMax(_dataSeriesContour1.MinXValue(), _dataSeriesContour1.MaxXValue());
            _plotChartContour1.Axes.Depth.SetMinMax(_dataSeriesContour1.MinYValue(), _dataSeriesContour1.MaxYValue());
            _plotChartContour1.Axes.Left.SetMinMax(_dataSeriesContour1.MinZValue(), _dataSeriesContour1.MaxZValue());

            // Set the drawing pen
            _dataSeriesContour1.Pen.Visible = false;

            // Set up the colour palette for the dislay
            setPalettContour(_dataSeriesContour1, _dataSeriesContour1.YValues.Minimum, _dataSeriesContour1.YValues.Maximum);

            // Set the chart cursor
            _plotChartContour1.Cursor = System.Windows.Forms.Cursors.Cross;

            // Set the control name and add the chart control to the user control base control
            this._tlpGrid.Controls.Add(_plotChartContour1, 0, 0);

            // Create the contour plot details to display below it
            Label details1 = new Label();
            details1.Dock = DockStyle.Fill;
            details1.ForeColor = Color.Blue;
            details1.Text = "Start time: " + _lowerTimeLimit1.ToString();
            details1.Text = details1.Text + " End time: " + _upperTimeLimit1.ToString() + "\n";
            details1.Text = details1.Text + "Start frequency: " + _lowerFrequencyLimit.ToString();
            details1.Text = details1.Text + " End frequency: " + _upperFrequencyLimit.ToString() + "\n";
            details1.Text = details1.Text + "Data Points: " + _value1.LongLength.ToString();
            
            this._tlpGrid.Controls.Add(details1, 0, 1);
        }

        /// <summary>
        /// Display contour plot 1
        /// </summary>
        private void displayContour2()
        {
            //
            // Local Declarations
            //

            //
            // Start of code
            //
            // Create the TChart
            _plotChartContour2 = new Steema.TeeChart.TChart();

            // Create the contour plot, create the Contour series
            _dataSeriesContour2 = new ColorGrid();

            // Turn off the data marks
            _dataSeriesContour2.Marks.Symbol.Shadow.Height = 1;
            _dataSeriesContour2.Marks.Symbol.Shadow.Visible = true;
            _dataSeriesContour2.Marks.Symbol.Shadow.Width = 1;
            _dataSeriesContour2.Marks.Visible = false;

            // Set the seiries title.
            _dataSeriesContour2.Title = "contourSeries";

            // Contour X, Y and Z Values
            _dataSeriesContour2.XValues.DataMember = "X";
            _dataSeriesContour2.YValues.DataMember = "Y";
            _dataSeriesContour2.ZValues.DataMember = "Z";

            // Add the data marks tip tool create the markstip
            _markTipContour2 = new Steema.TeeChart.Tools.MarksTip();

            // Add ,arkstip tool to the chart.
            _plotChartContour2.Tools.Add(_markTipContour2);

            // Set the style for label values
            _markTipContour2.Style = MarksStyles.PointIndex;
            _markTipContour2.MouseAction = Steema.TeeChart.Tools.MarksTipMouseAction.Click;
            _markTipContour2.MouseDelay = 100;
            _markTipContour2.HideDelay = 15000;

            // Make the waterfall series the one assiocated with the mark tip
            _markTipContour2.Series = _dataSeriesContour2;

            // Set the mark tip event handler.
            _markTipContour2.GetText += new Steema.TeeChart.Tools.MarksTipGetTextEventHandler(contour2MarkTipGetText);

            // Set the Axis for automatic scalling
            _plotChartContour2.Axes.Left.Automatic = true;
            _plotChartContour2.Axes.Left.AdjustMaxMin();
            _plotChartContour2.Axes.Bottom.Automatic = true;
            _plotChartContour2.Axes.Bottom.AdjustMaxMin();
            _plotChartContour2.Axes.Depth.Automatic = true;
            _plotChartContour2.Axes.Depth.AdjustMaxMin();
            _plotChartContour2.Axes.DepthTop.Automatic = true;

            // Set to fill available space
            _plotChartContour2.Dock = DockStyle.Fill;

            // Make the contour plot a 2D one.
            _plotChartContour2.Aspect.View3D = false;

            // Set the drawing mode
            _plotChartContour2.Aspect.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.HighQuality;
            _plotChartContour2.Aspect.TextRenderingHint = System.Drawing.Text.TextRenderingHint.ClearTypeGridFit;

            // Set the chart title
            _plotChartContour2.Header.Text = "Contour Plot 2: " + _time2.Length + " Data Points";

            // Set the Y (left) axis.
            _plotChartContour2.Axes.Left.Title.Text = ZAXIS_TITLE;

            // Set up the X (bottom) axis
            _plotChartContour2.Axes.Bottom.Labels.Font.Shadow.Visible = false;
            _plotChartContour2.Axes.Bottom.Labels.Shadow.Visible = true;

            // Set the X (bottom) axis caption
            _plotChartContour2.Axes.Bottom.Title.Text = XAXIS_TITLE;
            _plotChartContour2.Axes.Bottom.Title.Font.Shadow.Visible = false;
            _plotChartContour2.Axes.Bottom.Title.Font.Unit = System.Drawing.GraphicsUnit.World;
            _plotChartContour2.Axes.Bottom.Title.Shadow.Visible = false;

            // Add sample data to plot
            _plotChartContour2.Series.Add(_dataSeriesContour2);

            // Add the data with gaps to the data series
            _dataSeriesContour2.Add(_frequency2, _value2, _time2);

            // Set for irregularGrid
            _dataSeriesContour2.IrregularGrid = true;
            _dataSeriesContour2.CenteredPoints = false;

            // Set the axis minimum and maximum values
            _plotChartContour2.Axes.Bottom.SetMinMax(_dataSeriesContour2.MinXValue(), _dataSeriesContour2.MaxXValue());
            _plotChartContour2.Axes.Depth.SetMinMax(_dataSeriesContour2.MinYValue(), _dataSeriesContour2.MaxYValue());
            _plotChartContour2.Axes.Left.SetMinMax(_dataSeriesContour2.MinZValue(), _dataSeriesContour2.MaxZValue());

            // Set the drawing pen
            _dataSeriesContour2.Pen.Visible = false;

            // Set up the colour palette for the dislay
            setPalettContour(_dataSeriesContour2, _dataSeriesContour2.YValues.Minimum, _dataSeriesContour2.YValues.Maximum);

            // Set the chart cursor
            _plotChartContour2.Cursor = System.Windows.Forms.Cursors.Cross;

            // Set the control name and add the chart control to the user control base control
            this._tlpGrid.Controls.Add(_plotChartContour2, 1, 0);

            // Create the contour plot details to display below it
            Label details2 = new Label();
            details2.Dock = DockStyle.Fill;
            details2.ForeColor = Color.Blue;
            details2.Text = "Start time: " + _lowerTimeLimit2.ToString();
            details2.Text = details2.Text + " End time: " + _upperTimeLimit2.ToString() + "\n";
            details2.Text = details2.Text + "Start frequency: " + _lowerFrequencyLimit.ToString();
            details2.Text = details2.Text + " End frequency: " + _upperFrequencyLimit.ToString() + "\n";
            details2.Text = details2.Text + "Data Points: " + _value2.LongLength.ToString();
            
            this._tlpGrid.Controls.Add(details2, 1, 1);
        }
        #endregion
        
        #region Palette Operations
        /// <summary>
        /// Set up the palette colours based on 64 steps, min value blue (RGB blue 255), mid
        /// value green (RGB green 255) max value red (RGB red 255), i.e:
        /// 
        ///     min value to max-min/2 RGB blue 255 to 0 RGB green 0 to 255;
        ///     max-min/2 to max value RGB green 255 to 0 RGB red 0 to 255;
        /// 
        /// </summary>
        /// <param name="min"></param>
        /// <param name="max"></param>
        private void setPalettContour(Steema.TeeChart.Styles.ColorGrid dataSeriesContour, double min, double max)
        {
            //
            // Local Declarations
            //
            GridPalette paletteEntry;
            double value;
            double step;
            int red;
            int green;
            int blue;
            int colourStep;

            //
            // Start of Operation
            //
            step = (max - min) / PALETTEENTRIES;
            colourStep = 256 / (PALETTEENTRIES / 2);

            red = 0;
            blue = 255 + colourStep;
            green = -colourStep;
            value = min;

            dataSeriesContour.Palette.Clear();
            for (int entry = 0; entry < PALETTEENTRIES; entry++)
            {
                if (entry < PALETTEENTRIES / 2)
                {
                    blue = ((blue > colourStep) ? (blue - colourStep) : (0));
                    green = ((green < (255 - colourStep)) ? (green + colourStep) : (255));
                    red = 0;
                }
                else
                {
                    blue = 0;
                    green = ((green > colourStep) ? (green - colourStep) : (0));
                    red = ((red < (255 - colourStep)) ? (red + colourStep) : (255));
                }

                value = value + step;
                if (entry == PALETTEENTRIES - 1)
                {
                    value = max;
                }

                // Create a new palette entry
                paletteEntry = new GridPalette(value, Color.FromArgb(red, green, blue));
                dataSeriesContour.Palette.Add(paletteEntry);
            }

            // Request us of _dataSeries.Palette
            dataSeriesContour.UsePalette = true;
            dataSeriesContour.UseColorRange = false;
        }
        #endregion

        #region ColorGird (Contour Plot) mark text handlers
        /// <summary>
        /// Coutour plot 1 mark text handler
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void contour1MarkTipGetText(Steema.TeeChart.Tools.MarksTip sender, Steema.TeeChart.Tools.MarksTipGetTextEventArgs e)
        {
            //
            // Local Declarations
            //
            Steema.TeeChart.Styles.ColorGrid grid;

            //
            // Start of Operation
            //
            grid = (ColorGrid)sender.Series;
            int idx = int.Parse(e.Text);
            e.Text = "Index1: " + e.Text + "(" + idx.ToString() + ")\n" + 
                     grid.YValues[idx].ToString() + "g\n" +
                     grid.XValues[idx].ToString() + "Hz\n" +
                     "at " + grid.ZValues[idx].ToString() + "s";
        }

        /// <summary>
        /// Coutour plot 1 mark text handler
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void contour2MarkTipGetText(Steema.TeeChart.Tools.MarksTip sender, Steema.TeeChart.Tools.MarksTipGetTextEventArgs e)
        {
            //
            // Local Declarations
            //
            Steema.TeeChart.Styles.ColorGrid grid;

            //
            // Start of Operation
            //
            grid = (ColorGrid)sender.Series;
            int idx = int.Parse(e.Text);
            e.Text = "Index2: " + e.Text + "(" + idx.ToString() + ")\n" + 
                     grid.YValues[idx].ToString() + "g\n" +
                     grid.XValues[idx].ToString() + "Hz\n" +
                     "at " + grid.ZValues[idx].ToString() + "s";
        }
        #endregion
    }
}